;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc6c6d831e314faf01e332c0d643d2bd5 . "./code.cg2js/cg-common-control.fasl")
(#x3d2610cbf95a47dae60880e07a6db362 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5117ee49cd59a0e3675a091822951fd4 . "./code.cg2js/cg-yes-no-string.fasl")
(#x656cbf5290fff6f673113925390c869b . "./code.cg2js/cg-clipboard.fasl")
(#xdfabc1a9bcb58ba9d5e076454ff801e6 . "./code.cg2js/cg-dde-utils.fasl")
(#x09093f7ee066285b5cf8424a68e20c1a . "./code.cg2js/cg-editable-text.fasl")
(#xe2213cff8ae47f8a5a646ca2ddae8b19 . "./code.cg2js/cg-player.fasl")
(#x1cd81759e92c332bab6bd2c1ec995cb4 . "./code.cg2js/cg-text-or-combo.fasl")
(#xadb212140136c6bb336488567c455158 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x31673b643772e45d7c9e4baee2de93c2 . "./code.cg2js/cg-graphics-context.fasl")
(#xfe95ad4e095803639581f9e038756457 . "./code.cg2js/cg-choose-printer.fasl")
(#x1561c1dfd08a41d71164effb48bc6cf7 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe3b1c493f3166f07e3c2b6b0dd3089e0 . "./code.cg2js/cg-edit-in-place.fasl")
(#x3ece72ab80fea856ef38b061a6faf151 . "./code.cg2js/cg-split-bar.fasl")
(#xaecb8d9b206afa7bcb08816f73d8811a . "./code.cg2js/cg-timer.fasl")
(#x4d473376671ae697eff945f8654a24b9 . "./code.cg2js/cg-progress-indicator.fasl")
(#x0ff28167327e85fc4699dd0d12e29f74 . "./code.cg2js/cg-object-editor.fasl")
(#xc9a5bb2602857d004c8737135a434073 . "./code.cg2js/cg-fill-texture.fasl")
(#xd48c4643d7aa1bf18f903dc3eeb315f3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xd680736625093e91d9dd0bbc47f73d64 . "./code.cg2js/cg-os-window.fasl")
(#x7c032bd9990d8c4edabf439f1cce1eab . "./code.cg2js/cg-color-dialog.fasl")
(#x931822be586113d99b76ce7ae82279c2 . "./code.cg2js/cg-combo-box.fasl")
(#x03f8bec58139ce38eb349411c8954c10 . "./code.cg2js/cg-directory-dialog.fasl")
(#xead5d7f71163922058d8da7ee277d541 . "./code.cg2js/cg-tooltip.fasl")
(#xf7a7a544f998cc4302f55e1cce5670d7 . "./code.cg2js/cg-lettered-menu.fasl")
(#x2a71db84f32f7ab917640d882ddf2a30 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2a04bbd93d82e261dcb1e27492c3c1bc . "./code.cg2js/cg-pixmap.fasl")
(#x2d9dd55bb36b72827bf182695d23f7ad . "./code.cg2js/cg.fasl")
(#x76f34bc1b9616768e13c06fe0f55b732 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x1df0aebac243a7e379211652173aaad8 . "./code.cg2js/cg-template-string.fasl")
(#xc87e211034e426c53d813c2e922b7676 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa6b108d8889b0695da4aaa95abb91ee3 . "./code.cg2js/cg-check-box.fasl")
(#xd3ffa90729cb0ce4ca002e225f0297ce . "./code.cg2js/cg-toggling-widget.fasl")
(#x17260fd22d8b3e1c539ece498c2c9956 . "./code.cg2js/cg-radio-button.fasl")
(#x6ff235dc8a793296d00afe4561e0695f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1a53069c44af1491f0eacd3da2d4cb18 . "./code.cg2js/cg-scaling-stream.fasl")
(#xd395872198ea725c9392ffa2955596c6 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xcbec19d85346c3e6ef1c0c3f04dd3156 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd59aef3a8701b58361c754d999e22852 . "./code.cg2js/cg-grid-widget.fasl")
(#xe1d6bee1729456ce5f5d196395bb9595 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x449f6c9a4236e32264c21a3ff21649c9 . "./code.cg2js/cg-item-list.fasl")
(#x78d92f0e3650ecb759092eb712a53f48 . "./code.cg2js/cg-paren-matching.fasl")
(#x140e9d623b5580bf6c4f9b840db0f7e2 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf698c635e3d1c6f2c7418544109c4785 . "./code.cg2js/cg-html-dialog.fasl")
(#xb5806d6c5fc38a037512a020a7117b5f . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3c739323af0d5cfafac368a61adff1b8 . "./code.cg2js/cg-project-window.fasl")
(#x22cadfc490557661e95d38e46a95980c . "./code.cg2js/cg-toolbar.fasl")
(#x28646c8d46ecad1916eb84249760a464 . "./code.cg2js/cg-caret.fasl")
(#x95eaaded11cb2e06829b0365ce1d76c2 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4bd4902220f3ea8c4c4c7656b91121ad . "./code.cg2js/cg-web-browser.fasl")
(#xc4fb99500505e00ed67777de89c64875 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2ebda9b5d6b8501da00b042606af36b1 . "./code.cg2js/cg-font-dialog.fasl")
(#x21cfe73514f38d7afd39114c056dca88 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa06289f118ec009cc9be5a3c751f7400 . "./code.cg2js/cg-yes-no-list.fasl")
(#x7a9a2d65b503399a46b7196c6a87eade . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x3b3fef9536cf8ac32641bd93e42d933a . "./code.cg2js/cg-lamp.fasl")
(#xf91468b840f63610a1a35e9cc1df1761 . "./code.cg2js/cg-drawable.fasl")
(#xd75a15cd967f4a0260fc395b9694aaa2 . "./code.cg2js/cg-choice-list.fasl")
(#x5e75149e7a6ec227ff62d7d63cc97ef0 . "./code.cg2js/cg-curve.fasl")
(#xa136bb57a6766a746f5ba63a5632b9cc . "./code.cg2js/cg-base.fasl")
(#xd392edc0217b0cad41613452d075d22f . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x399eb605b63b43c305ff5b3ff3f4d47f . "./code.cg2js/cg-printing.fasl")
(#xf8e89ca64292b63f1288a14868628375 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xbb238b5ca0c5836589de4abe1f0270b4 . "./code.cg2js/cg-hotspot.fasl")
(#xc85ce34d00690acf92ff615b38042c7a . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb205f2054319d7584444780a01646fea . "./code.cg2js/cg-utility-dialog.fasl")
(#xf50cc253689b52ff5fca35e067131a18 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xc03dc2e99a456d7535eb52c1ac9624b0 . "./code.cg2js/cg-comtab.fasl")
(#x45938a56185cae01c9eb86864f263ab9 . "./code.cg2js/cg-os-widget.fasl")
(#x1b25919e2d179112b6478e417db2c0f7 . "./code.cg2js/cg-icon.fasl")
(#x6d0b0e1e99e0370d4f79b19995db6a07 . "./code.cg2js/cg-dev.fasl")
(#x8e780c27dd6e5c802887a9d0b3f5d290 . "./code.cg2js/cg-calendar.fasl")
(#x4c766b1458f4cf255f2367c9c06af59d . "./code.cg2js/cg-group-box.fasl")
(#xd2e4f968ea5932f00a3b443cfe41e6e1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x819d18f1802dc109f5b4e5a84bb04a84 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xc28170e1e7e1d4e92abf72f92d6a7c92 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xa3bc76be5fd4f15512f8020dde89ad28 . "./code.cg2js/cg-nodes.fasl")
(#xc44a8687b0bf2ca24feecb09e98992d3 . "./code.cg2js/cg-intersect.fasl")
(#x9331a666f9d1c7688a190270451a19d3 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xaf50fd7187f9edb84a47b8eeef475f44 . "./code.cg2js/cg-class-support.fasl")
(#xd0faaa8d7357112e3a24b09b8a9496ec . "./code.cg2js/cg-dialog-item.fasl")
(#x1768b6b8a9c2abf5c7b808398039d638 . "./code.cg2js/cg-plot-widget.fasl")
(#x67d6343705da128ce7069b3bb2115fd0 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x5717d664071c922c1c443debd5343b03 . "./code.cg2js/cg-up-down-control.fasl")
(#x5f58ff5c110771c19233703fdc180208 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x33ef96a77c14950cc121a16f2b47dae1 . "./code.cg2js/cg-status-bar.fasl")
(#x68a5e02ddd41da793c33b296c4c1f808 . "./code.cg2js/cg-class-grid.fasl")
(#x39f6899bdf404afd746541b4b7e06075 . "./code.cg2js/cg-wrap-string.fasl")
(#x0cd631eefccbe1a06faed18d7f33380c . "./code.cg2js/cg-layout.fasl")
(#x36751a98b9e4f3d34a6850743712539e . "./code.cg2js/cg-lisp-widget.fasl")
(#x99000874f68468d5de8be4752264a52c . "./code.cg2js/cg-button.fasl")
(#xa4569c02a45f772999c63cfce47a4bd7 . "./code.cg2js/cg-string-dialog.fasl")
(#xb0e679c1d68d07900171a9f86ec2e8a7 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x92147010e5d0c9298672c84dd87519e4 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x24f2e8c0f1273d7b22fea742044f0849 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xa257ba782b4cf27c2daf5229a04257a7 . "./code.cg2js/cg-static-text.fasl")
(#x02f3329268301c4f34a74ada7f5d9c5d . "./code.cg2js/cg-outline.fasl")
(#xad12f6cb2735f80a86d8564a910d016b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x23de1107b34ae3562566c6447a053967 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xbdd056f89b05685d5457a36150c2c88e . "./code.cg2js/cg-picture-widget.fasl")
(#xc6500567063b79097293027c60f55e36 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x78ee17bdde8965888681366a09709215 . "./code.cg2js/cg-chart-widget.fasl")
(#x800939a6655298c7b62e4f56ed1bfe5e . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x58a3cedf518a1f38d8ec5893f5756746 . "./code.cg2js/cg-lisp-text.fasl")
(#xb1d97d9c6538480f7a5350761ebc18ad . "./code.cg2js/cg-palette.fasl")
(#xf6dfca7d368e630847b9fa5eda1f7cfc . "./code.cg2js/cg-property.fasl")
(#xa0b9e234ed11bfb5f53e8767aed6f945 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x06bd0ebbe74aba564c925d12286fbef1 . "./code.cg2js/cg-file-dialog.fasl")
(#x6a3e7b2e975a3890b50b70b98c2dd9dd . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xb6929db5aae3edba062c23c4c7bef82a . "./code.cg2js/cg-get-pixmap.fasl")
(#x0f2f42b95faa3d1678309c11a95c5189 . "./code.cg2js/cg-tray.fasl")
(#x5b0f31addd1c824382b6445502948446 . "./code.cg2js/cg-acache.fasl")
(#xb9ab496b8c95c5b8637b847e7ee5ed84 . "./code.cg2js/cg-message-dialog.fasl")
(#xf50269d3e0f2325ea88adaa35668eaff . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x254b2c707bc6822543c379a54a872177 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf4b86162499674726f6048b1b4194318 . "./code.cg2js/cg-tab-control.fasl")
(#xabbf3342b5da8be6ee825bed2e2ae7d9 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xbb12053bc9b771881122cc1adc4021fa . "./code.cg2js/cg-menu-tooltip.fasl")
(#x5e3850115029774881d0043c34f2d442 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x666ada5c0fa75394999126ed567bcd79 . "./code.cg2js/cg-menu.fasl")
(#x44b4f9a5db6534bd71e6c324c355465a . "./code.cg2js/cg-get-position.fasl")
(#x0f1be5c45ecef6f3c830fc71d7634dcc . "./code.cg2js/cg-trackbar.fasl")
(#x4f59252892b6981c5c10377bd363ec4d . "./code.cg2js/cg-selected-object.fasl")
(#xfd745229c26db8ff3b6ad036c031e128 . "./code.cg2js/cg-list-view.fasl")
(#x291784db8cf12c3cdb49d02990f2e5e3 . "./code.cg2js/cg-html-widget.fasl")
(#x41533473dfe0d5b2ef316829c0d12985 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x4df9a67b7f90f742c1e28f7bf659abe4 . "./code.cg2js/cg-text-widget.fasl")
(#xa10a38338011755a05e0cfb83ebbcbeb . "./code.idejs/ide-base.fasl")
(#x3cc0099df39058136a132ad2da7c26c3 . "./code.idejs/ide-process-browser.fasl")
(#xcae951b2bb70ff49da892fc523ea444f . "./code.idejs/ide-patch-dialog.fasl")
(#x638ba8f71087845124253780d6101e25 . "./code.idejs/ide-keyshort.fasl")
(#x25103e57b4d0d8d74a0dc37fb059df73 . "./code.idejs/ide-window-list.fasl")
(#x24480af40943dec71ad81319e35bc664 . "./code.idejs/ide.fasl")
(#x480637d060fe8124f8073748eb20c39b . "./code.idejs/ide-editor.fasl")
(#xe961f36afa0114f62d399dd5e71058e7 . "./code.idejs/ide-clipboard.fasl")
(#xda36b659b3f4e9e90da4ae62b922e30d . "./code.idejs/ide-trace.fasl")
(#x6fe927941f4b074c3ff4e679d100ad19 . "./code.idejs/ide-class-browser.fasl")
(#xe36ff8cbf7f181be3f492f833d22c4a7 . "./code.idejs/ide-grapher.fasl")
(#x7211571fbe20714c9a111cecfaf925d4 . "./code.idejs/ide-apropos.fasl")
(#x0942ac32f9d55d797780ebbd7a2b1888 . "./code.idejs/ide-navigator.fasl")
(#x892531c9f9680f0d952d7654bcd89839 . "./code.idejs/ide-find-definitions.fasl")
(#x5698e1849b1ef6f680857afd41767e78 . "./code.idejs/ide-project.fasl")
(#x8905636a30bae3887ac5221967b3e10f . "./code.idejs/ide-class-grapher.fasl")
(#x289ae6038666cfdd47a2c025435f8b0c . "./code.idejs/ide-acache.fasl")
(#x10f691dcffb179cd7809f2548b6930b8 . "./code.idejs/ide-find-in-files.fasl")
(#x48ff14b43c7b842c4369a2d7d01f5973 . "./code.idejs/ide-inspector.fasl")
(#x1d9220f1c33d0b878385aa0fb46910cb . "./code.idejs/ide-intro.fasl")
(#x52019ec2694f224fad5adc8489d527a4 . "./code.idejs/ide-package-list.fasl")
(#x93eb8ce8d86cd7d87c71fac52c2e883b . "./code.idejs/ide-dev.fasl")
(#x5a0ee563967fac69172da128012eacff . "./code.idejs/ide-form.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
